/*
 * signals/alarm.c --
 *
 *	Demonstration of interval timers and how to ignore signals
 *	coming from terminating child processes.
 *
 * gcc -std=c99 -Wall -o alarm alarm.c
 */

#define _POSIX_C_SOURCE 200809L
#define _BSD_SOURCE

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>

static void
handler(int signal)
{
    fprintf(stderr, "\n%s: %s\n", __func__, strsignal(signal));
}

int
main(int argc, char *argv[])
{
    struct sigaction sa;
    pid_t pid;

    sa.sa_handler = handler;
    sigemptyset(&sa.sa_mask);
    sa.sa_flags = 0;

    sigaction(SIGALRM, &sa, NULL);
    sigaction(SIGVTALRM, &sa, NULL);

    sa.sa_handler = SIG_IGN;
    sigaction(SIGCHLD, &sa, NULL);

    struct itimerval itv = { { 1, 0}, { 2, 0} };
    
    if (setitimer(ITIMER_REAL, &itv, NULL) == -1) {
	perror("setitimer()");
	return EXIT_FAILURE;
    }

    if (setitimer(ITIMER_VIRTUAL, &itv, NULL) == -1) {
	perror("setitimer()");
	return EXIT_FAILURE;
    }

    while (1) {
	int n = 0, r = rand() & 0xffffff; /* generally not a smart idea */
	for (int i = 0; i < r; i++) {
	    n = (n-1) * (n-1);
	}
	usleep(1000);
	pid = fork();
	if (pid == -1) {
	    perror("fork()");
	} else if (pid == 0) {
	    exit(1);
	} else {
	    printf(".");
	}
    }
    
    return EXIT_SUCCESS;
}
